package com.sun.corba.se.impl.activation;

import java.io.ObjectOutputStream;
import java.io.FileOutputStream;
import java.util.Hashtable;
import java.util.Properties;
import java.util.Vector;
import com.sun.corba.se.spi.activation.ServerAlreadyUninstalled;
import com.sun.corba.se.spi.activation.ServerAlreadyInstalled;
import com.sun.corba.se.spi.activation.ServerNotRegistered;
import com.sun.corba.se.spi.activation.BadServerDefinition;
import com.sun.corba.se.spi.transport.SocketOrChannelAcceptor;
import java.util.Enumeration;
import com.sun.corba.se.spi.activation.ServerAlreadyRegistered;
import com.sun.corba.se.spi.activation.RepositoryPackage.ServerDef;
import java.io.ObjectInputStream;
import java.io.FileInputStream;
import java.io.File;
import com.sun.corba.se.impl.logging.ActivationSystemException;
import com.sun.corba.se.spi.orb.ORB;
import java.io.Serializable;
import com.sun.corba.se.spi.activation._RepositoryImplBase;

public class RepositoryImpl extends _RepositoryImplBase implements Serializable
{
    private static final long serialVersionUID = 8458417785209341858L;
    private transient boolean debug;
    static final int illegalServerId = -1;
    private transient RepositoryImpl.RepositoryDB db;
    transient ORB orb;
    transient ActivationSystemException wrapper;
    
    RepositoryImpl(ORB orb, File parent, boolean debug) {
        this.debug = false;
        this.db = null;
        this.orb = null;
        this.debug = debug;
        this.orb = orb;
        this.wrapper = ActivationSystemException.get(orb, "orbd.repository");
        File file = new File(parent, "servers.db");
        if (!file.exists()) {
            (this.db = new RepositoryImpl.RepositoryDB(file)).flush();
        }
        else {
            try {
                ObjectInputStream objectInputStream = new ObjectInputStream(new FileInputStream(file));
                this.db = (RepositoryImpl.RepositoryDB)objectInputStream.readObject();
                objectInputStream.close();
            }
            catch (Exception ex) {
                throw this.wrapper.cannotReadRepositoryDb(ex);
            }
        }
        orb.connect(this);
    }
    
    private String printServerDef(ServerDef serverDef) {
        return "ServerDef[applicationName=" + serverDef.applicationName + " serverName=" + serverDef.serverName + " serverClassPath=" + serverDef.serverClassPath + " serverArgs=" + serverDef.serverArgs + " serverVmArgs=" + serverDef.serverVmArgs + "]";
    }
    
    public int registerServer(ServerDef serverDef, int i) throws ServerAlreadyRegistered {
        synchronized (this.db) {
            Enumeration<Object> elements = this.db.serverTable.elements();
            while (elements.hasMoreElements()) {
                RepositoryImpl.DBServerDef dbServerDef = (RepositoryImpl.DBServerDef)elements.nextElement();
                if (!serverDef.applicationName.equals(dbServerDef.applicationName)) {
                    continue;
                }
                if (this.debug) {
                    System.out.println("RepositoryImpl: registerServer called to register ServerDef " + this.printServerDef(serverDef) + " with " + ((i == -1) ? "a new server Id" : ("server Id " + i)) + " FAILED because it is already registered.");
                }
                throw new ServerAlreadyRegistered(dbServerDef.id);
            }
            int incrementServerIdCounter;
            if (i == -1) {
                incrementServerIdCounter = this.db.incrementServerIdCounter();
            }
            else {
                incrementServerIdCounter = i;
            }
            this.db.serverTable.put(new Integer(incrementServerIdCounter), new RepositoryImpl.DBServerDef(serverDef, incrementServerIdCounter));
            this.db.flush();
            if (this.debug) {
                if (i == -1) {
                    System.out.println("RepositoryImpl: registerServer called to register ServerDef " + this.printServerDef(serverDef) + " with new serverId " + incrementServerIdCounter);
                }
                else {
                    System.out.println("RepositoryImpl: registerServer called to register ServerDef " + this.printServerDef(serverDef) + " with assigned serverId " + incrementServerIdCounter);
                }
            }
            return incrementServerIdCounter;
        }
    }
    
    public int registerServer(ServerDef serverDef) throws ServerAlreadyRegistered, BadServerDefinition {
        switch (new ServerTableEntry(this.wrapper, -1, serverDef, ((SocketOrChannelAcceptor)this.orb.getLegacyServerSocketManager().legacyGetEndpoint("BOOT_NAMING")).getServerSocket().getLocalPort(), "", true, this.debug).verify()) {
            case 0: {
                return this.registerServer(serverDef, -1);
            }
            case 1: {
                throw new BadServerDefinition("main class not found.");
            }
            case 2: {
                throw new BadServerDefinition("no main method found.");
            }
            case 3: {
                throw new BadServerDefinition("server application error.");
            }
            default: {
                throw new BadServerDefinition("unknown Exception.");
            }
        }
    }
    
    public void unregisterServer(int i) throws ServerNotRegistered {
        Integer n = new Integer(i);
        synchronized (this.db) {
            if ((RepositoryImpl.DBServerDef)this.db.serverTable.get(n) == null) {
                if (this.debug) {
                    System.out.println("RepositoryImpl: unregisterServer for serverId " + i + " called: server not registered");
                }
                throw new ServerNotRegistered();
            }
            else {
                this.db.serverTable.remove(n);
                this.db.flush();
            }
        }
        if (this.debug) {
            System.out.println("RepositoryImpl: unregisterServer for serverId " + i + " called");
        }
    }
    
    private RepositoryImpl.DBServerDef getDBServerDef(int value) throws ServerNotRegistered {
        RepositoryImpl.DBServerDef dbServerDef = (RepositoryImpl.DBServerDef)this.db.serverTable.get(new Integer(value));
        if (dbServerDef == null) {
            throw new ServerNotRegistered(value);
        }
        return dbServerDef;
    }
    
    public ServerDef getServer(int i) throws ServerNotRegistered {
        RepositoryImpl.DBServerDef dbServerDef = this.getDBServerDef(i);
        ServerDef serverDef = new ServerDef(dbServerDef.applicationName, dbServerDef.name, dbServerDef.classPath, dbServerDef.args, dbServerDef.vmArgs);
        if (this.debug) {
            System.out.println("RepositoryImpl: getServer for serverId " + i + " returns " + this.printServerDef(serverDef));
            return serverDef;
        }
        return serverDef;
    }
    
    public boolean isInstalled(int n) throws ServerNotRegistered {
        return this.getDBServerDef(n).isInstalled;
    }
    
    public void install(int n) throws ServerNotRegistered, ServerAlreadyInstalled {
        RepositoryImpl.DBServerDef dbServerDef = this.getDBServerDef(n);
        if (dbServerDef.isInstalled) {
            throw new ServerAlreadyInstalled(n);
        }
        dbServerDef.isInstalled = true;
        this.db.flush();
        return;
    }
    
    public void uninstall(int n) throws ServerNotRegistered, ServerAlreadyUninstalled {
        RepositoryImpl.DBServerDef dbServerDef = this.getDBServerDef(n);
        if (!dbServerDef.isInstalled) {
            throw new ServerAlreadyUninstalled(n);
        }
        dbServerDef.isInstalled = false;
        this.db.flush();
        return;
    }
    
    public int[] listRegisteredServers() {
        synchronized (this.db) {
            int n = 0;
            int[] array = new int[this.db.serverTable.size()];
            Enumeration<Object> elements = this.db.serverTable.elements();
            while (elements.hasMoreElements()) {
                array[n++] = ((RepositoryImpl.DBServerDef)elements.nextElement()).id;
            }
            if (this.debug) {
                StringBuffer o = new StringBuffer();
                for (int i = 0; i < array.length; ++i) {
                    o.append(' ');
                    o.append(array[i]);
                }
                System.out.println("RepositoryImpl: listRegisteredServers returns" + o.toString());
            }
            return array;
        }
    }
    
    public int getServerID(String s) throws ServerNotRegistered {
        synchronized (this.db) {
            int intValue = -1;
            Enumeration<Object> keys = this.db.serverTable.keys();
            while (keys.hasMoreElements()) {
                Integer key = (Integer)keys.nextElement();
                if (((RepositoryImpl.DBServerDef)this.db.serverTable.get(key)).applicationName.equals(s)) {
                    intValue = (int)key;
                    break;
                }
                continue;
            }
            if (this.debug) {
                System.out.println("RepositoryImpl: getServerID for " + s + " is " + intValue);
            }
            if (intValue == -1) {
                throw new ServerNotRegistered();
            }
            return intValue;
        }
    }
    
    public String[] getApplicationNames() {
        synchronized (this.db) {
            Vector<RepositoryImpl.DBServerDef> vector = new Vector();
            Enumeration<Object> keys = this.db.serverTable.keys();
            while (keys.hasMoreElements()) {
                RepositoryImpl.DBServerDef dbServerDef = (RepositoryImpl.DBServerDef)this.db.serverTable.get((Object)keys.nextElement());
                if (!dbServerDef.applicationName.equals("")) {
                    vector.addElement(dbServerDef.applicationName);
                }
                else {
                    continue;
                }
            }
            String[] array = new String[vector.size()];
            for (int i = 0; i < vector.size(); ++i) {
                array[i] = (String)vector.elementAt(i);
            }
            if (this.debug) {
                StringBuffer o = new StringBuffer();
                for (int j = 0; j < array.length; ++j) {
                    o.append(' ');
                    o.append(array[j]);
                }
                System.out.println("RepositoryImpl: getApplicationNames returns " + o.toString());
            }
            return array;
        }
    }
    
    public static void main(String[] array) {
        boolean n = false;
        for (int i = 0; i < array.length; ++i) {
            if (array[i].equals("-debug")) {
                n = true;
            }
        }
        try {
            Properties o = new Properties();
            o.put("org.omg.CORBA.ORBClass", "com.sun.corba.se.impl.orb.ORBImpl");
            ORB orb = (ORB)ORB.init(array, o);
            RepositoryImpl o2 = new RepositoryImpl(orb, new File(System.getProperty("com.sun.CORBA.activation.db", "db")), (boolean)n);
            orb.run();
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
    class RepositoryDB implements Serializable
    {
        File db;
        Hashtable serverTable;
        Integer serverIdCounter;
        
        RepositoryDB(File db) {
            this.db = db;
            this.serverTable = new Hashtable(255);
            this.serverIdCounter = new Integer(256);
        }
        
        int incrementServerIdCounter() {
            int intValue = (int)this.serverIdCounter;
            this.serverIdCounter = new Integer(++intValue);
            return intValue;
        }
        
        void flush() {
            try {
                this.db.delete();
                ObjectOutputStream objectOutputStream = new ObjectOutputStream(new FileOutputStream(this.db));
                objectOutputStream.writeObject(this);
                objectOutputStream.flush();
                objectOutputStream.close();
            }
            catch (Exception ex) {
                throw RepositoryImpl.this.wrapper.cannotWriteRepositoryDb(ex);
            }
        }
    }
    
    class DBServerDef implements Serializable
    {
        String applicationName;
        String name;
        String classPath;
        String args;
        String vmArgs;
        boolean isInstalled;
        int id;
        
        @Override
        public String toString() {
            return "DBServerDef(applicationName=" + this.applicationName + ", name=" + this.name + ", classPath=" + this.classPath + ", args=" + this.args + ", vmArgs=" + this.vmArgs + ", id=" + this.id + ", isInstalled=" + this.isInstalled + ")";
        }
        
        DBServerDef(ServerDef serverDef, int id) {
            this.applicationName = serverDef.applicationName;
            this.name = serverDef.serverName;
            this.classPath = serverDef.serverClassPath;
            this.args = serverDef.serverArgs;
            this.vmArgs = serverDef.serverVmArgs;
            this.id = id;
            this.isInstalled = false;
        }
    }
}
